<?php

class RedisSSIQueue {

  public function createItem($object_type, $object_id, $weight = 0) {
    $query = db_insert('redis_ssi_queue')
      ->fields(array(
        'object_id' => (string) $object_id,
        'object_type' => $object_type,
        // We cannot rely on REQUEST_TIME because many items might be created
        // by a single request which takes longer than 1 second.
        'created' => time(),
      ));
    try {
      $query->execute();
      return TRUE;
    }
    catch (PDOException $e) {
      // Ignore unique key violation errors. 23505 is the standard SQLSTATE,
      // which MySQL does not obey, so we also check for the MySQL specific
      // error code 1062.
      if ($e->getCode() != 23505 && $e->errorInfo[1] != 1062) {
        throw new PDOException($e);
      }
    }
  }

  public function claimItem($lease_time = 30) {
    return $this->claimItemWithType($lease_time);
  }
  public function claimPageItem($lease_time = 30) {
    return $this->claimItemWithType($lease_time, 'page');
  }
  protected function claimItemWithType($lease_time, $object_type = NULL) {
    $select = db_select('redis_ssi_queue', 'rsq')
      ->fields('rsq', array('object_type', 'object_id', 'item_id'))
      ->condition('expire', 0)
      // object type must come first to avoid regenerating the same pages over
      // and over again.
      ->orderBy('object_type')
      ->orderBy('weight')
      ->orderBy('item_id')
      ->range(0, 1);
    if (isset($object_type)) {
      $select->condition('object_type', $object_type);
    }
    while (TRUE) {
      $item = $select->execute()->fetchObject();
      if ($item) {
        $update = db_update('redis_ssi_queue')
          ->fields(array(
            'expire' => time() + $lease_time,
          ))
          ->condition('item_id', $item->item_id)
          ->condition('expire', 0);
        if ($update->execute()) {
          return $item;
        }
      }
      else {
        return FALSE;
      }
    }
  }
  public function releaseItem($item) {
    $update = db_update('redis_ssi_queue')
      ->fields(array(
        'expire' => 0,
      ))
      ->condition('item_id', $item->item_id);
      return $update->execute();
  }

  public function deleteItem($item) {
    db_delete('redis_ssi_queue')
      ->condition('item_id', $item->item_id)
      ->execute();
  }
}
